<?php

namespace Core\Factory;

use App\Application;
use Core\Env\Env;
use Core\Exceptions\ExceptionHandler;
use Core\Exceptions\Exceptions\ClassNotFoundException;
use Core\Http\Request;
use Core\Routing\RouteDispatcher;

class BootstrapFactory
{
    private Request $request;

    /**
     * Handle Http core
     */
    public function handle(): void
    {
        // Load env
        Env::load();

        // Create request
        $this->request = app()->make(Request::class, [$_POST + $_FILES]);

        // Register exceptions handler for error reporting
        app()->make(ExceptionHandler::class)->register();

        // Load routes
        require '../config/routes.php';

        try {
            // Boot application
            app()->make(Application::class)->bootstrap();

            // Dispatch router
            app()->make(RouteDispatcher::class)->dispatch($this->request);
        } catch (\Exception $e) {
            exceptions()->handle($e);
        }
    }

    /**
     * @param string $abstract
     * @param mixed  $arguments
     * @return mixed
     */
    public function make(string $abstract, mixed $arguments = []): mixed
    {
        return $this->resolve($abstract, $arguments);
    }

    /**
     * @param string $abstract
     * @param mixed  $arguments
     * @return mixed|null
     */
    private function resolve(string $abstract, mixed $arguments = []): mixed
    {
        if (class_exists($abstract)) {
            try {
                $reflection = new \ReflectionClass($abstract);
                return $reflection->newInstanceArgs($arguments);
            } catch (\ReflectionException $e) {
                exceptions()->handle($e);
            }
        }

        exceptions()->make(ClassNotFoundException::class, sprintf("Class '%s' not found", $abstract));
    }

    /**
     * Get the request instance
     *
     * @return \Core\Http\Request
     */
    public function request(): Request
    {
        return $this->request;
    }

    /**
     * Get path to file/folder in resources folder
     *
     * @param string $path
     * @return string
     */
    public function resourcePath(string $path = ''): string
    {
        return "../resources/" . $path;
    }
}