<?php

namespace Core\Routing;

use Core\Exceptions\Exceptions\NotFoundHttpException;
use ReflectionMethod;
use ReflectionNamedType;

class RouteValidator
{
    /**
     * Validate parameters against the method signature.
     *
     * @param object $controller
     * @param string $action
     * @param array  $params
     * @return array
     * @throws \Core\Exceptions\Exceptions\NotFoundHttpException
     * @throws \ReflectionException
     */
    public static function resolve(object $controller, string $action, array $params): array
    {
        $reflection       = new ReflectionMethod($controller, $action);
        $methodParameters = $reflection->getParameters();
        $validatedParams  = [];

        foreach ($methodParameters as $methodParameter) {
            $paramName    = $methodParameter->getName();
            $paramType    = $methodParameter->getType();
            $isOptional   = $methodParameter->isOptional();
            $defaultValue = $isOptional ? $methodParameter->getDefaultValue() : null;

            if (!array_key_exists($paramName, $params)) {
                if ($isOptional) {
                    $validatedParams[$paramName] = $defaultValue;
                    continue;
                }
                throw new NotFoundHttpException(sprintf("Missing parameter '%s' for action '%s'", $paramName, $action));
            }

            $value                       = $params[$paramName];
            $validatedParams[$paramName] = self::validateType($paramName, $value, $paramType, $action);
        }

        return $validatedParams;
    }

    /**
     * Validate and cast a parameter based on its type.
     *
     * @param string                   $paramName
     * @param mixed                    $value
     * @param ReflectionNamedType|null $paramType
     * @param string                   $action
     * @return mixed
     * @throws \Core\Exceptions\Exceptions\NotFoundHttpException
     */
    private static function validateType(string $paramName, mixed $value, ?ReflectionNamedType $paramType, string $action): mixed
    {
        if (!$paramType || $paramType->allowsNull() && $value === null) {
            return $value; // No type or allows null, return as-is.
        }

        $typeName = $paramType->getName();

        switch ($typeName) {
            case 'int':
                if (!ctype_digit((string)$value)) {
                    throw new NotFoundHttpException(sprintf("Invalid type for parameter '%s'. Expected integer, got '%s'", $paramName, gettype($value)));
                }
                return (int)$value;

            case 'float':
                if (!is_numeric($value)) {
                    throw new NotFoundHttpException(sprintf("Invalid type for parameter '%s'. Expected float, got '%s'", $paramName, gettype($value)));
                }
                return (float)$value;

            case 'string':
                if (!is_string($value)) {
                    throw new NotFoundHttpException(sprintf("Invalid type for parameter '%s'. Expected string, got '%s'", $paramName, gettype($value)));
                }
                return $value;

            default:
                throw new NotFoundHttpException(sprintf("Unsupported type '%s' for parameter '%s'", $typeName, $paramName));
        }
    }
}